/* 
PSP FILE IO (REV 3)

Descripcion: Ejemplo de entrada y salida de archivos utilizando el PSPSDK
Por: CaptainCPS-X (2010)

Rev 1: No tiene manejo de salida.
Rev 2: Agregado el manejo de Inicio y de Salida usando botones (entre otros detalles).
Rev 3: Corregida la deteccion de fallos al abrir / escribir el archivo.

Tome de referencia esta pagina: http://www.psp-programming.com/forums/index.php?topic=68

*/ 

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <pspkernel.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>
#include <pspctrl.h>

#define printf pspDebugScreenPrintf

PSP_MODULE_INFO("pspfileio", 0x1000, 1, 1);

int flags			= PSP_O_RDWR | PSP_O_CREAT ;	// Flag de Lectura / Escritura y Crear archivo de no Existir.
SceMode mode		= 0777;							// Permiso CHMOD para todo tipo de accion.
const char file[]	= "ms0:\\pspfile.txt";			// Archivo a escribir / leer
char fbuffer[256];									// Buffer que se usara para rellenar de texto.

void PSPFileIO();									// Prototipo de la funcion principal.
int bFinished = 0;									// Variable para saber si se ha realizado la funcion principal.

int main(int argc, char *argv[]) {

	// Iniciamos la pantalla para debug / depurar
	pspDebugScreenInit();

	// debug
	printf("PSP FILE IO (REV. 3) - por CaptainCPS-X (2010) \n\n");

	// debug
	printf("Este programa abrira el archivo [ %s ] y si no existe lo creara en la Memory Stick del PSP. Luego escribira varias cadenas de texto como forma de prueba.\n\n", file);

	printf("PRECIONA \"X\" PARA CONTINUAR...\n\n");

	// Buffer data de botones	
	SceCtrlData pad_data;

	// Loop atento a botones
	for(;;) {

		// Leemos el buffer de botones, por si acaso se ha precionado alguno
		sceCtrlReadBufferPositive(&pad_data, 1);

		// Si se ha precionado "X" realicamos la funcion principal
		if(pad_data.Buttons & PSP_CTRL_CROSS)
		{
			// Si ya se realizo todo, continua el Loop
			if(bFinished) continue;

			// debug
			printf("Iniciando funcion principal...\n\n");

			// Funcion principal
			PSPFileIO();
			
			// debug
			printf("PRECIONA \"O\" PARA VOLVER AL XBM...\n");
		}

		// Si se ha precionado "O" salimos del programa
		if(pad_data.Buttons & PSP_CTRL_CIRCLE)
		{
			// debug
			printf("Saliendo del programa...\n");
			
			// Salir del programa
			sceKernelExitGame();
		}

	}

	return 0;
}

void PSPFileIO() {

	// Abrimos el archivo y nos devuelve el ID
	
	SceUID fp = sceIoOpen(file, flags, mode);

	if(fp < 0) {
		// Error al abrir / crear el archivo.
		printf("Error al abrir / crear [ %s ].\n\n", file);
		
		bFinished = 1;
		
		return;
	}

	// debug
	printf("[ %s ] abierto / creado correctamente.\n\n", file);

	// Llenamos el buffer con una cadena de texto.
	sprintf(fbuffer, "Hola Mundo!.\n");

	// Escribimos el buffer al archivo y nos devuelve la cantidad de bytes escritos.
	
	// debug
	printf("Escribiendo: %s", fbuffer);

	int flen = sceIoWrite(fp, fbuffer, strlen(fbuffer));

	// Llenamos el buffer con una cadena de texto nuevamente.
	sprintf(fbuffer, "%d bytes escritos.\n", flen);

	// Escribimos por ultima vez el buffer al archivo.
	
	// debug
	printf("Escribiendo: %s", fbuffer);
	
	sceIoWrite(fp, fbuffer, strlen(fbuffer));

	// Cerramos el archivo

	// debug
	printf("\nCerrando archivo.\n\n");

	sceIoClose(fp);

	bFinished = 1;

}